MODULE FlaechenInhalt;
  FROM InOut     IMPORT ReadCard, WriteString, WriteInt, WriteLn;
  FROM RealInOut IMPORT ReadReal, WriteFloat;
  FROM MathLib   IMPORT sqrt;


(*
 * -- Programm zur Berechnung des Flaecheninhalts elementarer 
 * -- geometrischer Figuren
 *)

CONST
  pi =  3.141592;

TYPE
  Position   =  RECORD
                  x, y :  REAL
                END;
  geoTyp     =  (Dreieck, Rechteck, Kreis, Ellipse);
  geoFigur   =  RECORD
                  pos :  Position;   (* Lage der Figur *)
                  CASE typ :  geoTyp OF
                    Dreieck  :  (* 3 Ecken *)
                                (* A             :  pos *)
                                posB, posC       :  Position      |
                    Rechteck :  (* 1 Ecken + 2 Seitenlaengen *)
                                (* A             :  pos *)
                                hoehe, breite    :  REAL          |
                    Kreis    :  (* Zentrum + Radius *)
                                (* c             :  pos *)
                                rad              :  REAL          |
                    Ellipse  :  (* 2 Brennpunkte + Achse *)
                                (* 1. Brennpunkt :  pos *)
                                F                :  Position;
                                ha               :  REAL 
                                     (* 2a = grosse Achse *)
                    ELSE        (* leer *)
                  END
                END;

VAR
  figur                   :  geoFigur;
  t                       :  CARDINAL;
  la, lb, lc, lp, flaeche :  REAL;


BEGIN
  (* 
   * -- ------------------- 
   * -- Eingabe einer Figur 
   * -- ------------------- 
   *)
  WriteString("Figur eingeben ... "); 
  WriteString("Typ: 0 (3Eck), 1 (4Eck), 2 (Kreis), 3 (Ellipse) :  "); 
  ReadCard(t); WriteLn;

  WITH figur DO
    typ := VAL(geoTyp, t);
    WriteString("Position eingeben : "); WriteLn;
    WITH pos DO
      WriteString(" x ?  "); ReadReal(x);
      WriteString(" y ?  "); ReadReal(y); WriteLn
    END;

    CASE typ OF
      Dreieck  :  (* Position = Eckpunkt A *)
                  WriteString("Eckpunkte B u. C eingeben : "); WriteLn;
                  WITH posB DO
                    WriteString(" B "); 
                    WriteString(" x ?  "); ReadReal(x);
                    WriteString("    y ?  "); ReadReal(y); WriteLn
                  END;
                  WITH posC DO
                    WriteString(" C "); 
                    WriteString(" x ?  "); ReadReal(x);
                    WriteString("    y ?  "); ReadReal(y); WriteLn
                  END                                               |
      Rechteck :  (* Position = Eckpunkt A *)
                  WriteString("Hoehe u. Breite eingeben : "); WriteLn;
                  WriteString(" hoehe  ?  "); ReadReal(hoehe);
                  WriteString(" breite ?  "); ReadReal(breite);
                  WriteLn                                           |
      Kreis    :  (* Position = Mittelpunkt *)
                  WriteString("Radius eingeben : "); WriteLn;
                  WriteString(" radius  ?  "); ReadReal(rad);
                  WriteLn                                           |
      Ellipse  :  (* Position = 1. Brennpunkt *)
                  WriteString("2. Brennpunkt eingeben : "); WriteLn;
                  WITH F DO
                    WriteString(" F  "); WriteLn;
                    WriteString(" x ?  "); ReadReal(x);
                    WriteString(" y ?  "); ReadReal(y); WriteLn
                  END;
                  WriteString("Achse eingeben : "); WriteLn;
                  WriteString(" ha  ?  "); ReadReal(ha); WriteLn
      ELSE        WriteString("** unbekannter Figurtyp ! **"); WriteLn
    END (* CASE *)
  END; (* WITH *)

  (* 
   * -- ------------------------------------------ 
   * -- Berechnen der Flaeche der gewaehlten Figur 
   * -- ------------------------------------------ 
   *)
  WITH figur DO
    CASE typ OF
      Dreieck  :  (* Heronische Flaechenformel: 
                   *       flaeche = sqrt(p (p-a) (p-b) (p-c)) 
                   *   mit p = 1/2 (a + b + c)                 *)
                  la := sqrt((pos.x - posB.x)*(pos.x - posB.x) + 
                             (pos.y - posB.y)*(pos.y - posB.y));
                  lb := sqrt((posB.x - posC.x)*(posB.x - posC.x) + 
                             (posB.y - posC.y)*(posB.y - posC.y));
                  lc := sqrt((posC.x - pos.x)*(posC.x - pos.x) + 
                             (posC.y - pos.y)*(posC.y - pos.y));
                  lp := 0.5 * (la + lb + lc);
                  flaeche := sqrt(lp * (lp-la) * (lp-lb) * (lp-lc)) |
      Rechteck :  flaeche := hoehe * breite                         |
      Kreis    :  flaeche := pi * rad * rad                         |
      Ellipse  :  lc := 0.5 * sqrt((pos.x - F.x)*(pos.x - F.x) + 
                                   (pos.y - F.y)*(pos.y - F.y));
                  lb := sqrt(ha*ha - lc*lc);
                  flaeche := pi * ha * lb
      ELSE        flaeche := 0.0; 
                  WriteString("** unbekannter Figurtyp ! **"); 
                  WriteString("(flaeche = "); WriteFloat(flaeche, 4, 2); 
                  WriteString(")"); WriteLn
    END (* CASE *)
  END; (* WITH *)

  (* 
   * -- --------------------------------------- 
   * -- Ausgabe der Figur (Parameter + Flaeche)
   * -- --------------------------------------- 
   *)
  WITH figur DO
    WriteString("Position = ");
    WITH pos DO
      WriteString("("); WriteFloat(x, 3, 2); WriteString(","); 
                        WriteFloat(y, 3, 2); WriteString(")"); WriteLn
    END;

    CASE typ OF
      Dreieck  :  WITH posB DO
                    WriteString(" Ecke B  = ("); 
                    WriteFloat(x, 3, 2); WriteString(","); 
                    WriteFloat(y, 3, 2); WriteString(")"); WriteLn
                  END;
                  WITH posC DO
                    WriteString(" Ecke C  = ("); 
                    WriteFloat(x, 3, 2); WriteString(","); 
                    WriteFloat(y, 3, 2); WriteString(")"); WriteLn
                  END;
                  WriteString("Dreiecks-Flaeche = "); 
                  WriteFloat(flaeche, 4, 2); WriteLn                   |
      Rechteck :  WriteString(" Hoehe  = "); 
                  WriteFloat(hoehe, 3, 2); WriteLn;
                  WriteString(" Breite = "); 
                  WriteFloat(breite, 3, 2); WriteLn;
                  WriteString("Rechtecks-Flaeche = "); 
                  WriteFloat(flaeche, 4, 2); WriteLn                   |
      Kreis    :  WriteString(" Radius = ");
                  WriteFloat(rad, 3, 2); WriteLn;
                  WriteString("Kreis-Flaeche = "); 
                  WriteFloat(flaeche, 4, 2); WriteLn                   |
      Ellipse  :  WITH F DO
                    WriteString(" 2. Brennpunkt F = ("); 
                    WriteFloat(x, 3, 2); WriteString(","); 
                    WriteFloat(y, 3, 2); WriteString(")"); WriteLn
                  END;
                  WriteString(" Achse ha = "); 
                  WriteFloat(ha, 3, 2); WriteLn;
                  WriteString("Ellipsen-Flaeche = "); 
                  WriteFloat(flaeche, 4, 2); WriteLn
      ELSE        WriteString("** unbekannter Figurtyp ! **"); WriteLn
    END (* CASE *)
  END (* WITH *)
END FlaechenInhalt.
