MODULE KoeffFelder;
  FROM InOut     IMPORT WriteString, WriteLn;
  FROM RealInOut IMPORT WriteFloat;
  FROM MathLib   IMPORT exp, sqrt;

CONST
  KMin =  -10;
  KMax =   10;

TYPE
  KERNEL =  ARRAY [KMin..KMax] OF REAL;
  FUNC   =  PROCEDURE(REAL, REAL) :  REAL;


PROCEDURE box(
  x   :  REAL;
  rad :  REAL) :  REAL;

BEGIN
  IF ABS(x) <= rad THEN
    RETURN 1.0
  ELSE
    RETURN 0.0
  END
END box;


PROCEDURE gauss(
  x     :  REAL;
  sigma :  REAL) :  REAL;

CONST
  pi =  3.14159;

BEGIN
  IF x <= 3.0 * sigma THEN
    RETURN ( 1.0 / (sqrt(2.0 * pi) * sigma) * 
             exp(- x * x / (2.0 * sigma * sigma)) )
  ELSE
    RETURN 0.0
  END
END gauss;


PROCEDURE D1gauss(
  x     :  REAL;
  sigma :  REAL) :  REAL;

BEGIN
  RETURN ( - x / (sigma * sigma) * gauss(x, sigma) )
END D1gauss;


PROCEDURE zeroKernel(
  VAR kern :  ARRAY OF REAL);

VAR
  i :  CARDINAL;

BEGIN
  FOR i := 0 TO HIGH(kern) DO
    kern[i] := 0.0
  END
END zeroKernel;


PROCEDURE makeKernel(
  VAR kern     :  ARRAY OF REAL; 
      min, max :  INTEGER;
      genFunc  :  FUNC;
      param    :  REAL);

VAR
  i, len :  INTEGER;

BEGIN
  len := HIGH(kern) + 1;
  IF (len < max - min + 1) THEN
    max := len DIV 2;
    min := -max
  END;

  FOR i := min TO max DO
    kern[i - min] := genFunc(FLOAT(i), param)
  END
END makeKernel;


PROCEDURE PrintKernel(
  kern :  ARRAY OF REAL);

VAR
  i, len :  CARDINAL;

BEGIN
  len := HIGH(kern) + 1;
  FOR i := 0 TO (len - 1) DIV 2 - 1 DO
    WriteFloat(kern[i], 3, 3)
  END;
  WriteString(" |"); 
  WriteFloat(kern[(len - 1) DIV 2], 3, 3); 
  WriteString(" |");
  FOR i := (len - 1) DIV 2 + 1 TO len - 1 DO
    WriteFloat(kern[i], 3, 3)
  END;
  WriteLn
END PrintKernel;


VAR
  ff                                    :  PROC;
  funcBox, funcGauss, funcD1Gauss       :  FUNC;
  kernelBox, kernelGauss, kernelD1Gauss :  KERNEL;
  boxRad, sigma                         :  REAL;


BEGIN (* -- Hauptprogramm -- *)
  ff          := WriteLn;
  funcBox     := box;
  funcGauss   := gauss;
  funcD1Gauss := D1gauss;

  zeroKernel(kernelBox);
  zeroKernel(kernelGauss);
  zeroKernel(kernelD1Gauss);

  boxRad :=  5.0;
  sigma  :=  1.5;
  makeKernel(kernelBox, KMin, KMax, funcBox, boxRad);
  makeKernel(kernelGauss, KMin, KMax, funcGauss, sigma);
  makeKernel(kernelD1Gauss, KMin, KMax, funcD1Gauss, sigma);

  WriteString("Koeffizienten in Listen :  "); ff; ff;
  PrintKernel(kernelBox); ff;
  PrintKernel(kernelGauss); ff;
  PrintKernel(kernelD1Gauss); ff
END KoeffFelder.
