MODULE graphic;

   FROM MainWin IMPORT Clear, Flush, SetPos, Write, Read, WriteString, WriteLn, ReadInt;
   FROM RandomGenerator IMPORT Random;

   CONST
      dim = 20;

   VAR
      z: ARRAY [1..dim] OF INTEGER;
      num: INTEGER;

   PROCEDURE DisplayArray(a: ARRAY OF INTEGER; n: INTEGER);
      VAR
         i: INTEGER;
         ch: CHAR;
   BEGIN
      Clear;
      FOR i := 0 TO n-1 DO
         SetPos(a[i], i);
         Write('*');
         SetPos(i, n);
         Write('|');
         SetPos(n, i);
         Write('-');
      END;
      SetPos(n, n);
      Write('+');
      WriteLn;
      WriteString("Weiter mit RETURN.");
      WriteLn;
      Flush;
      Read(ch); (* warten auf RETURN *)
   END DisplayArray;


   PROCEDURE InsertionSort(VAR a: ARRAY OF INTEGER; n: INTEGER);
      VAR
         i, j, h: INTEGER;
   BEGIN
      DisplayArray(a, n);
      FOR i := 1 TO n-1 DO
         j := i;
         h := a[i];
         WHILE (j>0) AND (a[j-1]>h) DO
            a[j] := a[j-1];
            DEC(j);
         END;
         a[j] := h;
         DisplayArray(a, n);
      END;
   END InsertionSort;

   PROCEDURE SelectionSort(VAR a: ARRAY OF INTEGER; n: INTEGER);
      VAR
         i, j, h: INTEGER;
   BEGIN
      DisplayArray(a, n);
      FOR i := 0 TO n-2 DO
         h := i;
         FOR j := i+1 TO n-1 DO
            IF a[h]>a[j] THEN
               h := j; 
            END;
         END; 
         j := a[i];
         a[i] := a[h];
         a[h] := j;
         DisplayArray(a, n);
      END;
   END SelectionSort;

   PROCEDURE ShellSort(VAR a: ARRAY OF INTEGER; n: INTEGER);
      VAR
         i, j, h, s: INTEGER;
   BEGIN
      DisplayArray(a, n);

      s := 1;
      WHILE 3*s+1 < n DO
         s := 3*s+1;
      END;

      WHILE s>=1 DO
         FOR i := s TO n-1 DO
            j := i;
            h := a[i];
            WHILE (j>=s) AND (a[j-s]>h) DO
               a[j] := a[j-s];
               DEC(j, s);
            END;
            a[j] := h;
            DisplayArray(a, n);
         END;
         s := s DIV 3;
      END;
   END ShellSort;

   PROCEDURE QuickSort(VAR a: ARRAY OF INTEGER; n: INTEGER);

      PROCEDURE InternalQuickSort(VAR a: ARRAY OF INTEGER; start, end: INTEGER);
         VAR
            l, r, m, h: INTEGER;
      BEGIN
         l := start;
         r := end;
         m := a[Random(l, r)]; 
         
         WHILE l<=r DO
            WHILE a[l]<m DO
               INC(l);
            END;
            WHILE a[r]>m DO
               DEC(r);
            END;

            IF l<=r THEN
               h := a[l];
               a[l] := a[r];
               a[r] := h;
               INC(l);
               DEC(r);
            END;
         END;
         DisplayArray(a, n);

         IF start<r THEN
            InternalQuickSort(a, start, r);
         END;

         IF l<end THEN
            InternalQuickSort(a, l, end);
         END;
      END InternalQuickSort;

   BEGIN
      DisplayArray(a, n);
      InternalQuickSort(a, 0, n-1);
   END QuickSort;

   PROCEDURE HeapSort(VAR a: ARRAY OF INTEGER; n: INTEGER);

      VAR
         i, h: INTEGER;

      PROCEDURE Up(VAR a: ARRAY OF INTEGER; i: INTEGER);
         VAR
            j, h: INTEGER;
      BEGIN
         WHILE i>0 DO
            j := (i-1) DIV 2;
            IF a[j]<a[i] THEN
               h := a[j];
               a[j] := a[i];
               a[i] := h; 
               i := j;
            ELSE
               RETURN;
            END;
         END;
      END Up;
 
      PROCEDURE BuildHeap(VAR a: ARRAY OF INTEGER; n: INTEGER);
         VAR
            i: INTEGER;
      BEGIN
         FOR i := 1 TO n-1 DO
            Up(a, i);
            DisplayArray(a, n);
         END;
      END BuildHeap;

      PROCEDURE Down(VAR a: ARRAY OF INTEGER; n: INTEGER);
         VAR
            i, j, h: INTEGER;
      BEGIN
         i := 0;
         j := 1;
         WHILE j<n DO
            IF j+1<n THEN
               IF (a[j]>=a[j+1]) & (a[j]>a[i]) THEN
                  h := a[i];
                  a[i] := a[j];
                  a[j] := h;
               ELSIF (a[j]<a[j+1]) & (a[j+1]>a[i]) THEN
                  INC(j);
                  h := a[i];
                  a[i] := a[j];
                  a[j] := h;
               ELSE
                  RETURN;
               END;
            ELSIF a[j]>a[i] THEN
               h := a[i];
               a[i] := a[j];
               a[j] := h;
            ELSE
               RETURN;
            END;
            i := j;
            j := 2*i+1;
         END;
      END Down;

   BEGIN
      DisplayArray(a, n);
      BuildHeap(a, n);
      FOR i := n-1 TO 1 BY -1 DO
         h := a[0];
         a[0] := a[i];
         a[i] := h;
         Down(a, i);
         DisplayArray(a, n);
      END;
   END HeapSort;

   PROCEDURE InitArray(VAR a: ARRAY OF INTEGER; n: INTEGER);
      VAR
         i, j: INTEGER;
   BEGIN
      FOR i := 0 TO n-1 DO
         j := Random(0, i);
         a[i] := a[j];
         a[j] := i;
      END;
   END InitArray;

BEGIN
   InitArray(z, dim);

   Clear;
   WriteString("1 - Insertion Sort");
   WriteLn;
   WriteString("2 - Selection Sort");
   WriteLn;
   WriteString("3 - Shell Sort");
   WriteLn;
   WriteString("4 - Quick Sort");
   WriteLn;
   WriteString("5 - Heap Sort");
   WriteLn;
  
   WriteLn;
   WriteString ("Welches Verfahren wollen sie testen (Nummer eingeben)? ");
   Flush;
   ReadInt(num);
   WriteLn;

   CASE num OF
   | 1:
      InsertionSort(z, dim);
   | 2:
      SelectionSort(z, dim);
   | 3:
      ShellSort(z, dim);
   | 4:
      QuickSort(z, dim);
   | 5:
      HeapSort(z, dim);
   ELSE
      WriteLn;
      WriteString("Falsche Eingabe!");
      WriteLn;
   END;
END graphic.
